"""
Processing of the .wf files
"""
import logging
import os
import os.path
import re
import WebFrame as WF
from WebFrame import template
from WebFrame import auth

class _Block:
    def __init__(self, method, regex):
        self.method = method.strip()
        self.regex_plain = "^"+regex.strip()+"$"
        self.regex = re.compile(self.regex_plain, re.I)
        self.params = {}

    def get_read_policy(self):
        if "read-policy" in self.params:
            return self.params["read-policy"]
        else:
            return None

    def get_write_policy(self):
        if "write-policy" in self.params:
            return self.params["write-policy"]
        else:
            return None
        
    def set(self, param, value):
        self.params[param] = value

    def get(self, param):
        return self.params[param]
        
    def match(self, req):
        if req.method != self.method and self.method != "ANY":
            return False
        if req.path_info == None:
            pi = ""
        else:
            pi = req.path_info
        return self.regex.match(pi)

    def get_module(self):
        module = self.get("module")
        if WF.get_opt("debug"):
            logging.debug("WF loading module '%s'", module)

        modpath = module.rsplit(".", 1)
        if len(modpath) == 1:
            return __import__(module)
        else:
            return __import__(module, {}, {}, [modpath[0]])

    def get_func(self):
        function = self.get("function")
        if WF.get_opt("debug"):
            logging.debug("WF running function '%s'", function)
        m = self.get_module()
        f = getattr(m, function)
        return f

    def __str__(self):
        return "Block method='%s' regex='%s'" % (self.method, self.regex_plain)


def _get_block(f):
    b = None
    line = f.readline()
    while line:
        line = line.strip()
        
        if line == "END":
            return b

        # Ignore comments and empty lines
        if line != "" and line[0] != "#":
            if b == None:
                # Must be blockstart
                try:
                    method, regex = line.split(" ", 1)
                except ValueError:
                    b = _Block(line, "")
                else:
                    b = _Block(method, regex)
            else:
                param, val = line.split("=", 1)
                b.set(param.strip(), val.strip())
        
        line = f.readline()
    return None


def _process_template(b, ctx):
    filename = b.get("file")
    if filename[0] != "/":
        path, trash = ctx.req.filename.rsplit("/", 1)
        filename = path+"/"+filename

    # Must be within docroot
    filename = os.path.realpath(filename)
    if not filename.startswith(ctx.req.docroot):
        return ctx.resp.access_denied()
    elif os.access(filename, os.R_OK):
        # Check if the template-file is there and readable
        ctx.resp.write(template.run_template(ctx, filename))
        return ctx.resp.stop_processing()
    else:
        logging.error("WF Template '%s' not found", filename)
        return ctx.resp.not_found()

def _process_block(b, ctx):
    # Check access
    auth.check_access(ctx, b.get_read_policy(), b.get_write_policy())
    
    how = b.get("type")
    if how == "template":
        return _process_template(b, ctx)
    elif how == "plain":
        return WF._run_plain(b.get_func(), ctx)
    elif how == "xml":
        return WF._run_xml(b.get_func(), ctx)
    elif how == "json":
        return WF._run_json(b.get_func(), ctx)
    else:
        logging.error("Don't know how to process wf block of type %s", how)

def process(file, ctx):
    if WF.get_opt("debug"):
        logging.debug("WF processor: File: %s Path: '%s'",
                      file, ctx.req.path_info)

    if not os.access(file, os.R_OK):
        return ctx.resp.access_denied()
    
    f = open(file, "r")
    b = _get_block(f)
    while b:
        logging.debug("Got %s", str(b))
        if b.match(ctx.req):
            f.close()
            return _process_block(b, ctx)
        b = _get_block(f)
    f.close()
    
    return ctx.resp.not_found()
